"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parikshan = void 0;
/** Set this property on functions wrapped by parikshan */
const measured = Symbol('measured');
/**
 * Wraps input function with performance measurement.
 * If the wrapped function returns a promise,
 * a finally handler will be attached to the promise and the duration will be reported once the finally handler is invoked
 * @param fn - input function
 * @param loc - location of function in original source code
 * @returns function wrapped with performance hooks
 */
const parikshan = (fn, loc) => {
    const processComplete = (name, start, args) => {
        const end = performance.now();
        const returnedAt = Date.now();
        const duration = end - start;
        const functionName = name.split(' ').pop();
        // @ts-ignore: perf-hooke.d.ts doesn't contain performance.measure definition with object as second argument
        performance.measure('parikshan', {
            start: start,
            end: end,
            detail: {
                functionName: (functionName === null || functionName === void 0 ? void 0 : functionName.length) ? functionName : 'anonymous',
                calledAt: new Date(returnedAt - duration),
                returnedAt: new Date(returnedAt),
                arguments: args.map(arg => {
                    try {
                        const strigifiedArg = typeof arg === 'string' ? arg : JSON.stringify(arg);
                        if (!strigifiedArg) {
                            throw new Error('Not a JSON value');
                        }
                        return strigifiedArg;
                    }
                    catch (e) {
                        return arg === null || arg === void 0 ? void 0 : arg.toString();
                    }
                }),
                location: loc,
            },
        });
    };
    if (typeof fn === 'function') {
        if (fn[measured])
            return fn[measured];
        const wrap = (...args) => {
            const start = performance.now();
            const result = fn(...args);
            if (typeof (result === null || result === void 0 ? void 0 : result.finally) === 'function') {
                return result.finally(() => {
                    processComplete(fn.name, start, args);
                    return result;
                });
            }
            processComplete(fn.name, start, args);
            return result;
        };
        Object.defineProperties(wrap, {
            [measured]: {
                configurable: false,
                enumerable: false,
                value: wrap,
            },
            length: {
                configurable: false,
                enumerable: true,
                value: fn.length,
            },
            name: {
                configurable: false,
                enumerable: true,
                value: `parikshan ${fn.name}`,
            },
        });
        Object.defineProperties(fn, {
            [measured]: {
                configurable: false,
                enumerable: false,
                value: wrap,
            },
        });
        return wrap;
    }
    else {
        return fn;
    }
};
exports.parikshan = parikshan;
// TODO: a better way to do this
/**
 * Checks function string for native code/this.
 * @param fn - any function
 * @returns true if function string contains native code/this variable
 */
exports.parikshan.usesThisOrNative = (fn) => {
    if (!fn[measured]) {
        return /(\[native code\]|this)/g.test(fn.toString());
    }
    else {
        return false;
    }
};
//# sourceMappingURL=parikshan.js.map